package de.heinerkuecker.heterogeneous_map._1_simple;

import java.util.Map;

/**
 * Interface fr eine heterogene
 * typsichere {@link Map}.
 * <br />
 * Ziel ist es, in dieser {@link Map}
 * Werte unterschiedlichen Typs
 * zu speichern, aber fr den
 * jeweiligen Key den korrekten
 * Typ des Wertes abzusichern.
 * <br />
 * Der Kern der Umsetzung der heterogenen Map
 * mit Java-Generics ist die Festlegung von
 * Typ-Parametern fr Schlssel und Wert
 * (Key-Value) im Scope der jeweiligen
 * put- und get-Methode, wodurch der
 * Wert-Typ V durch den rekursiv definierten
 * Schlssel-Typ K bestimmt wird.
 * <br />
 * blicherweise sollte das
 * Verbinden(Backing) mit einer
 * echten Map per Delegation
 * erfolgen.
 * <br />
 * Die implementierende Klasse sollte
 * final sein (TODO Begrndung)
 * und eine Member fr den Schlssel-Wert
 * besitzen.
 * < br />
 * TODO zustzlich erforderliche Methoden wie {@link Map#size}, {@link Map#isEmpty} usw hier nachrsten
 *
 * @author Heiner K&uuml;cker
 */
public interface HeterogeneousMap
{
	/**
	 * Gegenber der Typ-Paarung von
	 * Schlssel und Wert typsichere
	 * {@link Map#put}-Methode.
	 * <br />
	 *
	 * TODO eventuell Rckgabewert von {@link Map#put} zurckgeben
	 *
	 * @param <V> Type-Parameter im Scope dieser Methode damit der Wert-Typ V durch den Schlssel-Typ bestimmt wird, Wert-Typ
	 * @param key Schlssel
	 * @param value Wert
	 */
	public <V> void put(
			final HMapKey<V> key ,
			final V value );

	/**
	 * Gegenber der Typ-Paarung von
	 * Schlssel und Wert typsichere
	 * {@link Map#get}-Methode.
	 *
	 * @param <V> Type-Parameter im Scope dieser Methode damit der Wert-Typ V durch den Schlssel-Typ bestimmt wird, Wert-Typ
	 * @param key Schlssel
	 * @return Wert
	 */
	public <V> V get(
			final HMapKey<V> key );
}