package de.heinerkuecker.heterogeneous_map._1_simple.example;

import de.heinerkuecker.heterogeneous_map._1_simple.HeterogeneousMap;
import de.heinerkuecker.heterogeneous_map._1_simple.convenience_keys.DateValueHMapKey;
import de.heinerkuecker.heterogeneous_map._1_simple.convenience_keys.StringValueHMapKey;

/**
 * Beispiel fr die Benutzung der
 * {@link HeterogeneousMap}.
 *
 * @author Heiner K&uuml;cker
 */
public class HeterogeneousMapUse
{
	/**
	 * Key-Klasse fr Vorname
	 */
	static class VornameKey
	implements StringValueHMapKey
	{
        // auf die Implementierung der Methoden wird verzichtet, da lediglich die Compiler-Prfung demonstriert werden soll, nicht die Wirkungsweise zur Laufzeit
        // in der realen Implementierung ist eine Member mit dem Schlssel-Wert und die korrekte Implementierung der Methoden hashCode, equals und eventuell bei Verwendung einer TreeMap compareTo erforderlich
	}

	/**
	 * Key-Klasse fr Geburtstag
	 */
	static class GeburtstagKey
	implements DateValueHMapKey
	{
        // auf die Implementierung der Methoden wird verzichtet, da lediglich die Compiler-Prfung demonstriert werden soll, nicht die Wirkungsweise zur Laufzeit
        // in der realen Implementierung ist eine Member mit dem Schlssel-Wert und die korrekte Implementierung der Methoden hashCode, equals und eventuell bei Verwendung einer TreeMap compareTo erforderlich
	}

	/**
	 * @param args unused
	 */
	public static void main(
			final String[] args)
	{
		//da hier nur die Compiler-Prfung wirken soll, kann die Map ohne Probleme null sein
		final HeterogeneousMap map = null;

		final String vorname = map.get( new VornameKey() );

		java.util.Date geburtsDate = map.get( new GeburtstagKey() );

		// TODO dies darf nicht kompilieren:geburtsDate = map.get( new VornameKey() );

		// null ist aufgrund der Beschrnkungen der Sprache Java immer erlaubt und umgeht die Prfung
		// Vermeiden von null muss ber dynamische Tests oder andere Prfungen, zum Beispiel mit dem checker framework, abgesichert werden
		geburtsDate = map.get( null );
	}

}
