package de.heinerkuecker.heterogeneous_map._2_constrained_key.example;

import java.math.BigDecimal;

import de.heinerkuecker.heterogeneous_map._2_constrained_key.convenience_keys.BigDecimalValueConstrainedKeyHMapKey;
import de.heinerkuecker.heterogeneous_map._2_constrained_key.convenience_keys.DateValueConstrainedKeyHMapKey;
import de.heinerkuecker.heterogeneous_map._2_constrained_key.convenience_keys.StringValueConstrainedKeyHMapKey;

/**
 * Beispiel fr die Benutzung der
 * {@link de.heinerkuecker.heterogeneous_map._2_constrained_key.ConstrainedKeyHeterogeneousMap}.
 *
 * @author Heiner K&uuml;cker
 */
public class ConstrainedKeyHeterogeneousMapUse
{
    /**
     * Map-Klasse zur Abbildung Personal-Nummer
     * auf Vorname und Geburtstag.
     */
	abstract static class PersonalNrToVornameGeburtstagMap
	implements de.heinerkuecker.heterogeneous_map._2_constrained_key.ConstrainedKeyHeterogeneousMap<PersonalNrToVornameGeburtstagMap>
	{
		// auf die Implementierung der Methoden wird verzichtet, da lediglich die Compiler-Prfung demonstriert werden soll, nicht die Wirkungsweise zur Laufzeit
	}

	/**
	 * Key-Klasse fr Vorname
	 */
	static class VornameKey
	implements StringValueConstrainedKeyHMapKey<PersonalNrToVornameGeburtstagMap>
	{
        // auf die Implementierung der Methoden wird verzichtet, da lediglich die Compiler-Prfung demonstriert werden soll, nicht die Wirkungsweise zur Laufzeit
	    // in der realen Implementierung ist eine Member mit dem Schlssel-Wert und die korrekte Implementierung der Methoden hashCode, equals und eventuell bei Verwendung einer TreeMap compareTo erforderlich
	}

	/**
	 * Key-Klasse fr Geburtstag
	 */
	static class GeburtstagKey
	implements DateValueConstrainedKeyHMapKey<PersonalNrToVornameGeburtstagMap>
	{
        // auf die Implementierung der Methoden wird verzichtet, da lediglich die Compiler-Prfung demonstriert werden soll, nicht die Wirkungsweise zur Laufzeit
        // in der realen Implementierung ist eine Member mit dem Schlssel-Wert und die korrekte Implementierung der Methoden hashCode, equals und eventuell bei Verwendung einer TreeMap compareTo erforderlich
	}

    /**
     * Map-Klasse zur Abbildung Personal-Nummer
     * auf Gehalt/Lohn.
     */
	abstract static class PersonalNrToGehaltMap
	implements de.heinerkuecker.heterogeneous_map._2_constrained_key.ConstrainedKeyHeterogeneousMap<PersonalNrToGehaltMap>
	{
        // auf die Implementierung der Methoden wird verzichtet, da lediglich die Compiler-Prfung demonstriert werden soll, nicht die Wirkungsweise zur Laufzeit
        // in der realen Implementierung ist eine Member mit dem Schlssel-Wert und die korrekte Implementierung der Methoden hashCode, equals und eventuell bei Verwendung einer TreeMap compareTo erforderlich
	}

	/**
	 * Key-Klasse fr Gehalt
	 */
	static class GehaltKey
	implements BigDecimalValueConstrainedKeyHMapKey<PersonalNrToGehaltMap>
	{
        // auf die Implementierung der Methoden wird verzichtet, da lediglich die Compiler-Prfung demonstriert werden soll, nicht die Wirkungsweise zur Laufzeit
        // in der realen Implementierung ist eine Member mit dem Schlssel-Wert und die korrekte Implementierung der Methoden hashCode, equals und eventuell bei Verwendung einer TreeMap compareTo erforderlich
	}

	/**
	 * @param args unused
	 */
	public static void main(
			final String[] args)
	{
		//da hier nur die Compiler-Prfung wirken soll, kann die Map ohne Probleme null sein
		final PersonalNrToVornameGeburtstagMap personalNrToVornameGeburtstagMap = null;

		final String vorname = personalNrToVornameGeburtstagMap.get( new VornameKey() );

		java.util.Date geburtsDate = personalNrToVornameGeburtstagMap.get( new GeburtstagKey() );

		// TODO dies darf nicht kompilieren: geburtsDate = personalNrToVornameGeburtstagMap( new VornameKey() );

		// null ist aufgrund der Beschrnkungen der Sprache Java immer erlaubt und umgeht die Prfung
		// Vermeiden von null muss ber dynamische Tests oder andere Prfungen, zum Beispiel mit dem checker framework, abgesichert werden
		geburtsDate = personalNrToVornameGeburtstagMap.get( null );


		//da hier nur die Compiler-Prfung wirken soll, kann die Map ohne Probleme null sein
		final PersonalNrToGehaltMap personalNrToGehaltMap = null;

		BigDecimal gehalt = personalNrToGehaltMap.get( new GehaltKey() );

		// TODO dies darf nicht kompilieren: gehalt = personalNrToVornameGeburtstagMap.get( new GehaltKey() );

		// TODO dies darf nicht kompilieren: gehalt = personalNrToGehaltMap.get( new VornameKey() );
	}

}
